local MAX_PARTY_BUFFS = 16 -- 队友最多显示BUFF数量
local MAX_PARTY_DEBUFFS = 8 -- 队友最多显示DEBUFF数量
local MAX_PARTY_ROWS = 8 -- 一行最多数量
local	DISPEL_LIST = { -- 可驱散列表
	['DRUID'] = {
		['Poison'] = 1,
		['Curse'] = 1
	},
	['MAGE'] = {
		['Poison'] = 1,
	},
	['PALADIN'] = {
		['Magic'] = 1,
		['Disease'] = 1,
		['Poison'] = 1
	},
	['PRIEST'] = {
		['Magic'] = 1,
		['Disease'] = 1,
	},
	['SHAMAN'] = {
		['Disease'] = 1,
		['Poison'] = 1
	},
	['WARLOCK'] = {
		['Magic'] = 1,
	},
}
DISPEL_LIST = DISPEL_LIST[select(2, UnitClass('player'))]

local BC = CreateFrame('Frame')

-- Buff框架
function BC:createBuff(parent, name, unit)
	for i = 1, MAX_PARTY_BUFFS do
		local buff = CreateFrame('Button', name  .. i, parent)
		if math.fmod(i, MAX_PARTY_ROWS) == 1 then
			buff:SetPoint('TOPLEFT', 48, -14 - 18 * math.ceil(i / MAX_PARTY_ROWS))
		else
			buff:SetPoint('TOPLEFT', getglobal(name .. i - 1), 'TOPRIGHT', 1, 0)
		end

		buff.Icon = buff:CreateTexture(nil, 'ARTWORK')
		buff.Icon:SetAllPoints(buff)

		buff.Cooldown = CreateFrame('Cooldown', nil, buff, 'CooldownFrameTemplate')
		buff.Cooldown:SetReverse(true)

		buff.CooldownText = buff.Cooldown:CreateFontString(nil, 'OVERLAY')
		buff.CooldownText:SetFont(STANDARD_TEXT_FONT, 10, 'OUTLINE')
		buff.CooldownText:SetPoint('CENTER', buff.Icon)

		-- 鼠标提示
		buff:SetScript('OnEnter', function(self)
			if GameTooltip then
				GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
				GameTooltip:SetUnitBuff(unit, i)
			end
		end)
		buff:SetScript('OnLeave', function()
			GameTooltip:Hide()
		end)
	end
end

-- Debuff框架
function BC:createDebuff(parent, name, unit, hide)
	for i = 1, MAX_PARTY_DEBUFFS do
		local hide = hide
		if hide then --隐藏默认DEBUFF
			hide = getglobal(hide .. i)
			if hide then
				hide:Hide()
				hide:SetScript('OnShow', function(self)
					self:Hide()
				end)
			end
		end

		local debuff = CreateFrame('Button', name .. i, parent)
		debuff:SetSize(18, 18)

		debuff.Border = debuff:CreateTexture(nil, 'OVERLAY')
		debuff.Border:SetTexture('Interface\\Buttons\\UI-Debuff-Border')
		debuff.Border:SetTexCoord(0, 1, 0, 1)
		debuff.Border:SetPoint('CENTER', debuff)
		debuff.Border:SetSize(20, 20)

		debuff.Icon = debuff:CreateTexture(nil, 'ARTWORK')
		debuff.Icon:SetAllPoints(debuff)

		debuff.Cooldown = CreateFrame('Cooldown', nil, debuff, 'CooldownFrameTemplate')
		debuff.Cooldown:SetReverse(true)

		debuff.CooldownText = debuff.Cooldown:CreateFontString(nil, 'OVERLAY')
		debuff.CooldownText:SetFont(STANDARD_TEXT_FONT, 10, 'OUTLINE')
		debuff.CooldownText:SetPoint('CENTER', debuff.Icon)

		-- 鼠标提示
		debuff:SetScript('OnEnter', function(self)
			if GameTooltip then
				GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
				GameTooltip:SetUnitDebuff(unit, i)
			end
		end)
		debuff:SetScript('OnLeave',function()
			GameTooltip:Hide()
		end)
	end
end

-- Buff更新
function BC:updateBuff(name, unit)
	local count = 0
	for i = 1, MAX_PARTY_BUFFS do
		local buff = getglobal(name .. i)
		local _, icon, _, _, duration, expirationTime, caster = UnitBuff(unit, i)
		if icon then
			count = count  + 1
			buff:Show()
			buff.Icon:SetTexture(icon)
			CooldownFrame_Set(buff.Cooldown, expirationTime - duration, duration, true)
			if caster == 'player' or caster == 'pet' then -- 只显示自己/宠物施放的CD
				buff:SetSize(18, 18)
				local cd = math.ceil(expirationTime - GetTime())
				if cd > 0 then
					buff.CooldownText:SetTextColor(1, 1, 1)
					buff.CooldownText:SetFont(STANDARD_TEXT_FONT, 10, 'OUTLINE')
					if cd < 6 then
						buff.CooldownText:SetTextColor(1, 0, 0)
						buff.CooldownText:SetFont(STANDARD_TEXT_FONT, 17, 'OUTLINE')
					elseif cd < 10 then
						buff.CooldownText:SetTextColor(1, 1, 0)
						buff.CooldownText:SetFont(STANDARD_TEXT_FONT, 16, 'OUTLINE')
					elseif cd < 60 then
						buff.CooldownText:SetTextColor(1, 1, 0)
						buff.CooldownText:SetFont(STANDARD_TEXT_FONT, 15, 'OUTLINE')
					elseif cd <= 1800 then
						cd = math.ceil(cd/60) ..'m'
					else
						cd = math.ceil(cd/3600) ..'h'
					end
					buff.CooldownText:SetText(cd)
				else
					buff.CooldownText:SetText()
				end
			else
				buff.CooldownText:SetText()
				buff:SetSize(14, 14)
			end
		else
			buff:Hide()
		end
	end
	return count
end

-- Deuff更新
function BC:updateDeuff(name, unit, count)
	local rows = math.ceil(count / MAX_PARTY_ROWS) -- 行数
	for i = 1, MAX_PARTY_DEBUFFS do
		local debuff = getglobal(name .. i)
		local n, icon, _, debuffType, duration, expirationTime, _, isStealable = UnitDebuff(unit, i)
		if icon then
			rows = math.ceil(count / MAX_PARTY_ROWS) + math.ceil(i / MAX_PARTY_ROWS)
			if math.fmod(i, MAX_PARTY_ROWS) == 1 then
				debuff:SetPoint('TOPLEFT', 48, -14 - 18 * rows)
			else
				debuff:SetPoint('TOPLEFT', getglobal(name .. i - 1), 'TOPRIGHT', 1, 0)
			end
			debuff:Show()
			debuff.Icon:SetTexture(icon)
			CooldownFrame_Set(debuff.Cooldown, expirationTime - duration, duration, true)
			if not isStealable and DISPEL_LIST[debuffType] then -- 可以驱散
				local cd = math.ceil(expirationTime - GetTime())
				if cd > 0 then
					debuff.CooldownText:SetTextColor(1, 1, 1)
					debuff.CooldownText:SetFont(STANDARD_TEXT_FONT, 10, 'OUTLINE')
					if cd < 6 then
						debuff.CooldownText:SetTextColor(1, 0, 0)
						debuff.CooldownText:SetFont(STANDARD_TEXT_FONT, 17, 'OUTLINE')
					elseif cd < 10 then
						debuff.CooldownText:SetTextColor(1, 1, 0)
						debuff.CooldownText:SetFont(STANDARD_TEXT_FONT, 16, 'OUTLINE')
					elseif cd < 60 then
						debuff.CooldownText:SetTextColor(1, 1, 0)
						debuff.CooldownText:SetFont(STANDARD_TEXT_FONT, 15, 'OUTLINE')
					elseif cd <= 1800 then
						cd = math.ceil(cd/60) ..'m'
					else
						cd = math.ceil(cd/3600) ..'h'
					end
					debuff.CooldownText:SetText(cd)
				else
					debuff.CooldownText:SetText()
				end
			else
				debuff.CooldownText:SetText()
			end
		else
			debuff:Hide()
		end
	end
	return rows
end

for id = 1, MAX_PARTY_MEMBERS do
	BC:createBuff(getglobal('PartyMemberFrame'.. id), 'BC_PartyMemberFrame'.. id ..'Buff', 'party'.. id) -- 队友Buff
	BC:createBuff(getglobal('BC_Party'.. id ..'TargetFrame'), 'BC_PartyMemberFrame'.. id ..'TargetBuff', 'party'.. id ..'target') -- 队友目标Buff
	BC:createDebuff(getglobal('PartyMemberFrame'.. id), 'BC_PartyMemberFrame'.. id ..'Debuff', 'party'.. id, 'PartyMemberFrame'.. id .. 'Debuff') -- 队友Debuff
	BC:createDebuff(getglobal('BC_Party'.. id ..'TargetFrame'), 'BC_PartyMemberFrame'.. id ..'TargetDebuff', 'party'.. id ..'target', nil) -- 队友目标Debuff
end

--队友buff/debuff直接显示时隐藏buff鼠标提示
hooksecurefunc('PartyMemberBuffTooltip_Update', function(self)
	PartyMemberBuffTooltip:Hide()
end)

BC:SetScript('OnUpdate', function(self, elapsed)
	self.timer = (self.timer or 0) + elapsed
	if self.timer < 0.1 then return end
	self.timer = 0



	for id = 1, MAX_PARTY_MEMBERS do
		local unit = 'party'.. id
		local unittarget = unit ..'target'
		local rows = 0
		if UnitExists(unit) then
			rows = BC:updateBuff('BC_PartyMemberFrame'.. id ..'Buff', unit)
			rows = BC:updateDeuff('BC_PartyMemberFrame'.. id ..'Debuff', unit, rows)

			-- 施法条定位
			local spellBar = getglobal('BC_PartyMemberFrame'.. id ..'SpellBar')
			if spellBar and spellBar:IsVisible() then
				if rows < 1 then rows = 1 end
				local anchor, _, _, x, y = spellBar:GetPoint()
				y = - rows * 22
				spellBar:SetPoint(anchor, x, y)
			end
		end

		if UnitExists(unittarget) then
			rows = BC:updateBuff('BC_PartyMemberFrame'.. id ..'TargetBuff', unittarget)
			BC:updateDeuff('BC_PartyMemberFrame'.. id ..'TargetDebuff', unittarget, rows)
		end
	end
end)
